package data.campaign.econ.industries;

import java.util.Random;

import com.fs.starfarer.api.Global;
import com.fs.starfarer.api.campaign.BattleAPI;
import com.fs.starfarer.api.campaign.CampaignEventListener.FleetDespawnReason;
import com.fs.starfarer.api.campaign.CampaignFleetAPI;
import com.fs.starfarer.api.campaign.FactionAPI.ShipPickMode;
import com.fs.starfarer.api.campaign.econ.CommodityOnMarketAPI;
import com.fs.starfarer.api.campaign.econ.Industry;
import com.fs.starfarer.api.campaign.listeners.FleetEventListener;
import com.fs.starfarer.api.campaign.rules.MemoryAPI;
import com.fs.starfarer.api.impl.campaign.econ.impl.BaseIndustry;
import com.fs.starfarer.api.impl.campaign.econ.impl.MilitaryBase.PatrolFleetData;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactory.PatrolType;
import com.fs.starfarer.api.impl.campaign.fleets.FleetFactoryV3;
import com.fs.starfarer.api.impl.campaign.fleets.FleetParamsV3;
import com.fs.starfarer.api.impl.campaign.fleets.PatrolAssignmentAIV4;
import com.fs.starfarer.api.impl.campaign.fleets.RouteManager;
import com.fs.starfarer.api.impl.campaign.fleets.RouteManager.OptionalFleetData;
import com.fs.starfarer.api.impl.campaign.fleets.RouteManager.RouteData;
import com.fs.starfarer.api.impl.campaign.fleets.RouteManager.RouteFleetSpawner;
import com.fs.starfarer.api.impl.campaign.fleets.RouteManager.RouteSegment;
import com.fs.starfarer.api.impl.campaign.ids.Commodities;
import com.fs.starfarer.api.impl.campaign.ids.Industries;
import com.fs.starfarer.api.impl.campaign.ids.MemFlags;
import com.fs.starfarer.api.impl.campaign.ids.Ranks;
import com.fs.starfarer.api.impl.campaign.ids.Stats;
import com.fs.starfarer.api.impl.campaign.ids.Submarkets;
import com.fs.starfarer.api.impl.campaign.rulecmd.salvage.MarketCMD.RaidDangerLevel;
import com.fs.starfarer.api.ui.TooltipMakerAPI;
import com.fs.starfarer.api.util.IntervalUtil;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.util.Pair;
import com.fs.starfarer.api.util.WeightedRandomPicker;

public class MS_redWingsHQ extends BaseIndustry implements RouteFleetSpawner, FleetEventListener {
    
    public static float OFFICER_PROB = 0.15f;
    public static float DEFENSE_BONUS = 0.15f;
    
    public static int IMPROVE_NUM_PATROLS_BONUS = 1;
    
    @Override
    public boolean isHidden() {
	return !market.getFactionId().equals("shadow_industry");
    }
	
    @Override
    public boolean isFunctional() {
	return super.isFunctional() && market.getFactionId().equals("shadow_industry");
    }

    public void apply() {
	super.apply(true);
		
	int size = market.getSize();
		
	demand(Commodities.SUPPLIES, size - 1);
	demand(Commodities.FUEL, size - 1);
	demand(Commodities.SHIPS, size - 1);
		
	supply(Commodities.CREW, size);
		
	demand(Commodities.HAND_WEAPONS, size);
	supply(Commodities.MARINES, size);
			
	Pair<String, Integer> deficit = getMaxDeficit(Commodities.HAND_WEAPONS);
	applyDeficitToProduction(1, deficit, Commodities.MARINES);
		
	modifyStabilityWithBaseMod();
        
        int light = 2;
        int medium = 1;
        int heavy = 1;
        
        market.getStats().getDynamic().getMod(Stats.PATROL_NUM_LIGHT_MOD).modifyFlat(getModId(), light);
        market.getStats().getDynamic().getMod(Stats.PATROL_NUM_MEDIUM_MOD).modifyFlat(getModId(), medium);
        market.getStats().getDynamic().getMod(Stats.PATROL_NUM_HEAVY_MOD).modifyFlat(getModId(), heavy);
		
	MemoryAPI memory = market.getMemoryWithoutUpdate();
	Misc.setFlagWithReason(memory, MemFlags.MARKET_PATROL, getModId(), true, -1);
	//Misc.setFlagWithReason(memory, MemFlags.MARKET_MILITARY, getModId(), true, -1);
	
        if (isHidden()) {
            supply.clear();
            demand.clear();
            unapply();
        }
        
	if (!isFunctional()) {
            supply.clear();
            unapply();
	}
    }

    @Override
    public void unapply() {
	super.unapply();
		
	MemoryAPI memory = market.getMemoryWithoutUpdate();
	Misc.setFlagWithReason(memory, MemFlags.MARKET_PATROL, getModId(), false, -1);
	//Misc.setFlagWithReason(memory, MemFlags.MARKET_MILITARY, getModId(), false, -1);
		
	unmodifyStabilityWithBaseMod();
    }
	
    protected boolean hasPostDemandSection(boolean hasDemand, IndustryTooltipMode mode) {
	return mode != IndustryTooltipMode.NORMAL || isFunctional();
    }
	
    @Override
    protected void addPostDemandSection(TooltipMakerAPI tooltip, boolean hasDemand, IndustryTooltipMode mode) {
	if (mode != IndustryTooltipMode.NORMAL || isFunctional()) {
            addStabilityPostDemandSection(tooltip, hasDemand, mode);
	}
    }
	
    @Override
    protected int getBaseStabilityMod() {
        int stab = 0;
        if (market.getPrevStability() < 10) stab = 1;
        
        return stab;
    }
	
    public String getNameForModifier() {
	if (getSpec().getName().contains("Command")) {
            return getSpec().getName();
	}
	return Misc.ucFirst(getSpec().getName());
    }
	
    @Override
    protected Pair<String, Integer> getStabilityAffectingDeficit() {
	return getMaxDeficit(Commodities.SUPPLIES, Commodities.FUEL, Commodities.SHIPS, Commodities.HAND_WEAPONS);
    }
	
    @Override
    public String getCurrentImage() {
	return super.getCurrentImage();
    }

	
    public boolean isDemandLegal(CommodityOnMarketAPI com) {
	return true;
    }

    public boolean isSupplyLegal(CommodityOnMarketAPI com) {
	return true;
    }

    protected IntervalUtil tracker = new IntervalUtil(Global.getSettings().getFloat("averagePatrolSpawnInterval") * 0.7f,
									  Global.getSettings().getFloat("averagePatrolSpawnInterval") * 1.3f);
	
    protected float returningPatrolValue = 0f;
	
    @Override
    protected void buildingFinished() {
	super.buildingFinished();
		
	tracker.forceIntervalElapsed();
    }
	
    @Override
    protected void upgradeFinished(Industry previous) {
	super.upgradeFinished(previous);
		
	tracker.forceIntervalElapsed();
    }

    @Override
    public void advance(float amount) {
	super.advance(amount);
	
	if (Global.getSector().getEconomy().isSimMode() || !isFunctional()) return;
		
	float days = Global.getSector().getClock().convertToDays(amount);
	
        //the redwings are deployed less often, being kept as a special response unit
        //as such their spawn rate for general patrols should be lower
	float spawnRate = 0.5f;
	float rateMult = market.getStats().getDynamic().getStat(Stats.COMBAT_FLEET_SPAWN_RATE_MULT).getModifiedValue();
	spawnRate *= rateMult;
		
	float extraTime = 0f;
	if (returningPatrolValue > 0) {
            // apply "returned patrols" to spawn rate, at a maximum rate of 1 interval per day
            float interval = tracker.getIntervalDuration();
            extraTime = interval * days;
            returningPatrolValue -= days;
            if (returningPatrolValue < 0) returningPatrolValue = 0;
	}
	tracker.advance(days * spawnRate + extraTime);
		
	if (tracker.intervalElapsed()) {
            String sid = getRouteSourceId();
			
            int light = getCount(PatrolType.FAST);
            int medium = getCount(PatrolType.COMBAT);
            int heavy = getCount(PatrolType.HEAVY);

            int maxLight = 3;
            int maxMedium = 2;
            int maxHeavy = 1;
			
            WeightedRandomPicker<PatrolType> picker = new WeightedRandomPicker<>();
            picker.add(PatrolType.HEAVY, maxHeavy - heavy); 
            picker.add(PatrolType.COMBAT, maxMedium - medium); 
            picker.add(PatrolType.FAST, maxLight - light); 
			
            if (picker.isEmpty()) return;
			
            PatrolType type = picker.pick();
            PatrolFleetData custom = new PatrolFleetData(type);
			
            OptionalFleetData extra = new OptionalFleetData(market);
            extra.fleetType = type.getFleetType();
			
            RouteData route = RouteManager.getInstance().addRoute(sid, market, Misc.genRandomSeed(), extra, this, custom);
            float patrolDays = 35f + (float) Math.random() * 10f;
			
            route.addSegment(new RouteSegment(patrolDays, market.getPrimaryEntity()));
	}
    }
	
    public void reportAboutToBeDespawnedByRouteManager(RouteData route) {
    }
	
    public boolean shouldRepeat(RouteData route) {
	return false;
    }
	
    public int getCount(PatrolType ... types) {
	int count = 0;
	for (RouteData data : RouteManager.getInstance().getRoutesForSource(getRouteSourceId())) {
            if (data.getCustom() instanceof PatrolFleetData) {
		PatrolFleetData custom = (PatrolFleetData) data.getCustom();
		for (PatrolType type : types) {
                    if (type == custom.type) {
			count++;
			break;
                    }
		}
            }
	}
	return count;
    }

    public int getMaxPatrols(PatrolType type) {
	if (type == PatrolType.FAST) {
            return (int) market.getStats().getDynamic().getMod(Stats.PATROL_NUM_LIGHT_MOD).computeEffective(0);
	}
	if (type == PatrolType.COMBAT) {
            return (int) market.getStats().getDynamic().getMod(Stats.PATROL_NUM_MEDIUM_MOD).computeEffective(0);
	}
	if (type == PatrolType.HEAVY) {
            return (int) market.getStats().getDynamic().getMod(Stats.PATROL_NUM_HEAVY_MOD).computeEffective(0);
	}
	return 0;
    }
	
    public boolean shouldCancelRouteAfterDelayCheck(RouteData route) {
	return false;
    }

    public void reportBattleOccurred(CampaignFleetAPI fleet, CampaignFleetAPI primaryWinner, BattleAPI battle) {
		
    }

    public void reportFleetDespawnedToListener(CampaignFleetAPI fleet, FleetDespawnReason reason, Object param) {
	if (!isFunctional()) return;
		
	if (reason == FleetDespawnReason.REACHED_DESTINATION) {
            RouteData route = RouteManager.getInstance().getRoute(getRouteSourceId(), fleet);
            if (route.getCustom() instanceof PatrolFleetData) {
		PatrolFleetData custom = (PatrolFleetData) route.getCustom();
		if (custom.spawnFP > 0) {
                    float fraction  = fleet.getFleetPoints() / custom.spawnFP;
                    returningPatrolValue += fraction;
		}
            }
        }
    }
	
    public CampaignFleetAPI spawnFleet(RouteData route) {	
	PatrolFleetData custom = (PatrolFleetData) route.getCustom();
	PatrolType type = custom.type;
		
	Random random = route.getRandom();
		
	float combat = 0f;
	float tanker = 0f;
	float freighter = 0f;
	String fleetType = type.getFleetType();
	switch (type) {
	case FAST:
            combat = Math.round(3f + (float) random.nextFloat() * 2f) * 5f;
            break;
	case COMBAT:
            combat = Math.round(6f + (float) random.nextFloat() * 3f) * 5f;
            tanker = Math.round((float) random.nextFloat()) * 5f;
            break;
	case HEAVY:
            combat = Math.round(10f + (float) random.nextFloat() * 5f) * 5f;
            tanker = Math.round((float) random.nextFloat()) * 10f;
            freighter = Math.round((float) random.nextFloat()) * 10f;
            break;
	}
		
	FleetParamsV3 params = new FleetParamsV3(
			market, 
			null, // loc in hyper; don't need if have market
			"redwings",
			route.getQualityOverride(), // quality override
			fleetType,
			combat, // combatPts
			freighter, // freighterPts 
			tanker, // tankerPts
			0f, // transportPts
			0f, // linerPts
			0f, // utilityPts
			0f // qualityMod
			);
	params.timestamp = route.getTimestamp();
	params.random = random;
	params.modeOverride = Misc.getShipPickMode(market);
	params.modeOverride = ShipPickMode.PRIORITY_THEN_ALL;
	CampaignFleetAPI fleet = FleetFactoryV3.createFleet(params);
		
	if (fleet == null || fleet.isEmpty()) return null;
		
	fleet.setFaction(market.getFactionId(), true);
	fleet.setNoFactionInName(true);
		
	fleet.addEventListener(this);
		
	fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_PATROL_FLEET, true);
	fleet.getMemoryWithoutUpdate().set(MemFlags.FLEET_IGNORES_OTHER_FLEETS, true, 0.3f);

	if (type == PatrolType.FAST || type == PatrolType.COMBAT) {
            fleet.getMemoryWithoutUpdate().set(MemFlags.MEMORY_KEY_CUSTOMS_INSPECTOR, true);
	}
		
	String postId = Ranks.POST_PATROL_COMMANDER;
	String rankId = Ranks.SPACE_COMMANDER;
	switch (type) {
	case FAST:
            rankId = Ranks.SPACE_LIEUTENANT;
            break;
	case COMBAT:
            rankId = Ranks.SPACE_COMMANDER;
            break;
	case HEAVY:
            rankId = Ranks.SPACE_CAPTAIN;
            break;
	}
		
	fleet.getCommander().setPostId(postId);
	fleet.getCommander().setRankId(rankId);
	
	market.getContainingLocation().addEntity(fleet);
	fleet.setFacing((float) Math.random() * 360f);
	// this will get overridden by the patrol assignment AI, depending on route-time elapsed etc
	fleet.setLocation(market.getPrimaryEntity().getLocation().x, market.getPrimaryEntity().getLocation().y);
		
	fleet.addScript(new PatrolAssignmentAIV4(fleet, route));
		
	if (custom.spawnFP <= 0) {
            custom.spawnFP = fleet.getFleetPoints();
	}
		
	return fleet;
    }
	
    public String getRouteSourceId() {
	return getMarket().getId() + "_" + "redwings";
    }

    @Override
    public boolean isAvailableToBuild() {
	return false;
    }
	
    public boolean showWhenUnavailable() {
	return false;
    }

    @Override
    public boolean canImprove() {
	return false;
    }
	
    @Override
    public RaidDangerLevel adjustCommodityDangerLevel(String commodityId, RaidDangerLevel level) {
	return level.next();
    }

    @Override
    public RaidDangerLevel adjustItemDangerLevel(String itemId, String data, RaidDangerLevel level) {
    	return level.next();
    }
}